/* SQL schema for validation of APB-1 queries */

/* Floating-point quantities are defined with FLOAT in here. You may wish to substitute
 * a type such as REAL, depending on your RDBMS implementation of FLOATing-point types.
 */


/* ---------------------------------------------- */
/* DIMENSION TABLES                               */
/* ---------------------------------------------- */

CREATE TABLE ProdLevel
(
	Code_level  			char(12) NOT NULL,
	Class_level 			char(12) NOT NULL,
	Group_level 			char(12) NOT NULL,
	Family_level            char(12) NOT NULL,
	Line_level  			char(12) NOT NULL,
	Division_level          char(12) NOT NULL
)
;

CREATE TABLE ProdTree
(
	Member_name      char(12) NOT NULL,
	Parent_name      char(12) NULL,
	level_name       varchar(12) NOT NULL
)
;

CREATE TABLE CustLevel
(
	Store_level       char(12) NOT NULL,
	Retailer_level    char(12) NOT NULL
)
;

CREATE TABLE CustTree
(
	Member_name        		char(12) NOT NULL,
	Parent_name        		char(12) NULL,
	level_name 			varchar(12) NOT NULL
)
;

CREATE TABLE TimeLevel
(
	Month_level   			varchar(12) NOT NULL,
	Quarter_level   		varchar(12) NOT NULL,
	year_level  			varchar(12) NOT NULL
)
;

CREATE TABLE TimeTree
(
	Member_name        		varchar(12) NOT NULL,
	Parent_name        		varchar(12) NULL,
	level_name				varchar(12) NOT NULL
)
;

CREATE TABLE ChanLevel
(
	Base_level  	char(12) NOT NULL
	,all_level      char(12)
)
;

CREATE TABLE ChanTree
(
	Member_name        	char(12) NOT NULL,
	Parent_name        	char(12) NULL,
	level_name 			varchar(12) NOT NULL
)
;

/* ---------------------------------------------- */
/* FACT TABLES                                    */
/* ---------------------------------------------- */

create table actvars (
	Customer_level      	char(12) NOT NULL,
	Product_level       	char(12) NOT NULL,
	Channel_level       	char(12) NOT NULL,
	Time_level  			varchar(12) NOT NULL,
	UnitsSold     		FLOAT  NOT NULL,
	DollarSales   		FLOAT  NOT NULL,
	DollarCost   		FLOAT  NOT NULL
)
;

create table planvars (
	Customer_level		char(12) NOT NULL,
	Product_level		char(12) NOT NULL,
	Time_level			varchar(12) NOT NULL,
	UnitsSold     		FLOAT NOT NULL,
	DollarSales   		FLOAT NOT NULL,
    DollarCost			FLOAT NOT NULL
)
;

CREATE TABLE HistInventory
(
	Customer_level      		char(12) NOT NULL,
	Product_level       		char(12) NOT NULL,
	Inv199501		        smallint NOT NULL,
	Inv199502		        smallint NOT NULL,
	Inv199503		        smallint NOT NULL,
	Inv199504		        smallint NOT NULL,
	Inv199505		        smallint NOT NULL,
	Inv199506		        smallint NOT NULL,
	Inv199507		        smallint NOT NULL,
	Inv199508		        smallint NOT NULL,
	Inv199509		        smallint NOT NULL,
	Inv199510		        smallint NOT NULL,
	Inv199511		        smallint NOT NULL,
	Inv199512		        smallint NOT NULL,
	Inv199601		        smallint NOT NULL,
	Inv199602		        smallint NOT NULL,
	Inv199603		        smallint NOT NULL,
	Inv199604		        smallint NOT NULL,
	Inv199605		        smallint NOT NULL
)
;

CREATE TABLE CurrInventory
(
	Customer_level      		char(12) NOT NULL,
	Product_level       		char(12) NOT NULL,
	Inv199606		        smallint NULL
)
;

CREATE TABLE StdProdCost
(
	Product_level	char(12) NOT NULL,
	Scenario        varchar(8) NOT NULL,
	Time_level		varchar(12) NOT NULL,
	Value			FLOAT  NOT NULL
)
;

CREATE TABLE StdShipCost
(
	Customer_level     char(12) NOT NULL,
	Scenario            varchar(8) NOT NULL,
	Time_level  		varchar(12) NOT NULL,
	Value 			FLOAT NOT NULL
)
;


/* ---------------------------------------------- */
/* TEMPORARY & INTERMEDIATE TABLES                */
/* ---------------------------------------------- */

CREATE TABLE tmprolling
(
	product_level       	char(12) NOT NULL,
	customer_level      	char(12) NOT NULL,
	channel_level       	char(12) NOT NULL,
	time_level  			varchar(12) NOT NULL,
	dollarsales   		FLOAT NOT NULL
)
;

CREATE TABLE FcstRetailer1996Tmp
(
	Retailer_level     char(12) NOT NULL,
	UnitsSold     	FLOAT   NOT NULL,
	DollarSales   	FLOAT   NOT NULL,
	DollarCost  	FLOAT   NOT NULL 
)
;

create table Plan1996RetailerTmp (
	Retailer_level	char(12) NOT NULL,
	UnitsSold     	FLOAT  NOT NULL,
	DollarSales   	FLOAT  NOT NULL,
	DollarCost  	FLOAT  NOT NULL
)
;

CREATE TABLE ActYToD1996MonthStore
(
	Store_level 	char(12) NOT NULL,
	Time_level  	varchar(12) NOT NULL,
	UnitsSold     	FLOAT NOT NULL,
	DollarSales   	FLOAT NOT NULL,
	DollarCost  	FLOAT NOT NULL
)
;

create table ActYToD1996MonthRetailerTmp (
	Retailer_level	char(12) not null,
	Time_level		varchar(12) not null,
	UnitsSold		FLOAT not null,
	DollarSales 	FLOAT not null,
	DollarCost      FLOAT not null
)
;

CREATE TABLE FcstMonthStore
(
	Store_level 			char(12) NOT NULL,
	Time_level  			varchar(12) NOT NULL,
	UnitsSold     		FLOAT NOT NULL,
	DollarSales   		FLOAT  NOT NULL,
	DollarCost  			FLOAT  NOT NULL
)
;

CREATE TABLE FcstMonthRetailer1996
(
	Retailer_level 	char(12) NOT NULL,
	Time_level  	varchar(12) NOT NULL,
	UnitsSold   	FLOAT NOT NULL,
	DollarSales   	FLOAT NOT NULL,
	DollarCost		FLOAT NOT NULL
)
;

CREATE TABLE PlanYToD1996MonthStoreProduct
(
	Product_level   char(12) NOT NULL,
	Store_level 	char(12) NOT NULL,
	Time_level  	varchar(12) NOT NULL,
	UnitsSold		FLOAT NOT NULL,
	DollarSales   	FLOAT NOT NULL,
	DollarCost		FLOAT NOT NULL
)
;

create table PlanYToD1996MonthStoreTmp
(
	Store_level		char(12) not null,
	Time_level		varchar(12) not null,
	UnitsSold		FLOAT not null,
	DollarSales		FLOAT not null,
	DollarCost		FLOAT not null
)
;

CREATE TABLE FcstMonthStoreProduct
(
	Product_level   char(12) NOT NULL,
	Store_level 	char(12) NOT NULL,
	Time_level  	varchar(12) NOT NULL,
	UnitsSold     	FLOAT NOT NULL,
	DollarSales   	FLOAT  NOT NULL,
	DollarCost  	FLOAT  NOT NULL
)
;

create table q8a_tmp (
	product     char (12) not null,
	customer    char (12) not null,
	timeper     varchar (12) not null,
	dollarsales FLOAT  not null
)
;
create table q8b_tmp (
	product     char (12) not null,
	customer    char (12) not null,
	timeper     varchar (12) not null,
	dollarsales FLOAT  not null
)
;
create table q8c_tmp (
	product     char (12) not null,
	customer    char (12) not null,
	timeper     varchar (12) not null,
	dollarsales FLOAT  not null
)
;

create table q9a_tmp (
	product char (12) not null,
	customer char (12) not null,
	timeper varchar (12) not null,
	dollarsales FLOAT  not null
)
;

create table q9b_tmp (
	product char (12) not null,
	customer char (12) not null,
	timeper varchar (12) not null,
	dollarsales FLOAT  not null
)
;
create table q9c_tmp (
	product char (12) not null,
	customer char (12) not null,
	timeper varchar (12) not null,
	dollarsales FLOAT  not null
)
;


/* ---------------------------------------------- */
/* USEFUL INDEXES                                 */
/* ---------------------------------------------- */

CREATE INDEX PLF1 ON ProdLevel
(
	Code_level 
)
;

CREATE INDEX PLF2 ON ProdLevel
(
	Class_level 
)
;

CREATE INDEX PLF3 ON ProdLevel
(
	Group_level 
)
;

CREATE INDEX PLF4 ON ProdLevel
(
	Family_level 
)
;

CREATE INDEX PLF5 ON ProdLevel
(
	Line_level 
)
;

CREATE INDEX PLF6 ON ProdLevel
(
	Division_level 
)
;


CREATE INDEX PTM ON ProdTree
(
	Member_name 
)
;

CREATE INDEX PTP ON ProdTree
(
	Parent_name 
)
;

CREATE INDEX CTL1 ON CustLevel
(
	Store_level 
)
;
CREATE INDEX CTL2 ON CustLevel
(
	Retailer_level 
)
;

CREATE INDEX CTM ON CustTree
(
	Member_name 
)
;
CREATE INDEX CTP ON CustTree
(
	Parent_name 
)
;


/* timetree too small to usefully index */

/* timelevel too small to usefully index */


CREATE INDEX ProdIndexB ON PlanVars
(
	Customer_level 
)
;
CREATE INDEX CustIndexB ON PlanVars
(
	Product_level 
)
;
CREATE INDEX TimeIndexB ON PlanVars
(
	Time_level 
)
;

CREATE INDEX ProdIndex ON ActVars
(
	Customer_level 
)
;
CREATE INDEX CustIndex ON ActVars
(
	Product_level 
)
;
CREATE INDEX TimeIndex ON ActVars
(
	Time_level 
)
;
CREATE INDEX ChanIndex ON ActVars
(
	Channel_level 
)
;


CREATE INDEX StdProdCost_Keys ON StdProdCost
(
	Product_level ,
	Time_level ,
	Scenario 
)
;


CREATE INDEX StdShipCost_Keys ON StdShipCost
(
	Customer_level ,
	Time_level ,
	Scenario 
)
;


CREATE INDEX TMPPR ON tmprolling
(
	product_level 
)
;
CREATE INDEX TMPCUST ON tmprolling
(
	customer_level 
)
;
CREATE INDEX TMPTIME ON tmprolling
(
	time_level 
)
;
CREATE INDEX TMPCHAN ON tmprolling
(
	channel_level 
)
;

CREATE INDEX TFMS ON FcstMonthStore
(
	Time_level 
)
;
CREATE INDEX RFMS ON FcstMonthStore
(
	Store_level 
)
;

CREATE INDEX TFMR96 ON FcstMonthRetailer1996
(
	Time_level 
)
;
CREATE INDEX RFMR96 ON FcstMonthRetailer1996
(
	Retailer_level 
)
;

CREATE INDEX PFMSP ON FcstMonthStoreProduct
(
	Product_level 
)
;
CREATE INDEX TFMSP ON FcstMonthStoreProduct
(
	Time_level 
)
;
CREATE INDEX CUFMSP ON FcstMonthStoreProduct
(
	Store_level 
)
;


/* ---------------------------------------------- */
/* VIEWS                                          */
/* ---------------------------------------------- */

create view Fcst1996Retailer (
	Retailer_level,
	UnitsSold,
	DollarSales,
	DollarCost
)
as
select
	custlevel.retailer_level,
    (sum(a.unitssold)*2*1.15) as UnitsSold,
    (sum(a.dollarsales)*2*1.15) as DollarSales,
    (sum(a.DollarCost)*2*1.15) as DollarCost 
from
	actvars a, custlevel
where 
	    custlevel.store_level=a.customer_level
    and a.time_level in (
		'199507','199508','199509','199510','199511','199512'
	)
group by 
	custlevel.retailer_level
;

create view Plan1996Retailer (
	retailer_level,
	UnitsSold,
	DollarSales,
	DollarCost 
)
as
select 
    custlevel.retailer_level,
    sum(a.unitssold) as UnitsSold,
    sum(a.dollarsales) as DollarSales,
    sum(a.DollarCost) as DollarCost
from
	PlanVars a, custlevel
where
	    custlevel.store_level = a.customer_level
	and a.time_level = '1996'
group by 
	custlevel.retailer_level
;

create view ActYToD1996MonthRetailer (
	Retailer_level,
	time_level,
	UnitsSold,
	DollarSales,
	DollarCost 
)
as
select
	custlevel.retailer_level,
    a.time_level,
    sum(a.unitssold) as UnitsSold,
    sum(a.dollarsales) as DollarSales,
    sum(a.DollarCost) as DollarCost
from
	ActYToD1996MonthStore a, custlevel
where
	a.store_level=custlevel.store_level
group by
	custlevel.retailer_level,
    a.time_level
;

create view PlanYToD1996MonthStore (
	Store_level,
	Time_level,
	UnitsSold,
	DollarSales,
	DollarCost 
)
as
select
	a.store_level,
    a.time_level,
    sum(a.unitssold) as UnitsSold,
    sum(a.dollarsales) as DollarSales,
    sum(a.DollarCost) as DollarCost 
from
	PlanYToD1996MonthStoreProduct a
group by 
	a.store_level,
    a.time_level
;

/* end of SQL */
